﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.ServiceProcess;
using System.Text;

using System.IO;
using System.Xml;
using System.IO.Pipes;
using System.Threading;

namespace SecurityEventHandler
{
    public partial class SecurityEventHandler : ServiceBase
    {
        bool m_fLoop = true;
        Thread m_SvcThread = null;

        public SecurityEventHandler()
        {
            InitializeComponent();
        }

        protected override void OnStart(string[] args)
        {
            //
            //  Start a worker thread.
            //
            if (m_SvcThread == null)
            {
                m_SvcThread = new Thread(ThreadLoop);
                m_SvcThread.IsBackground = true;
                m_SvcThread.Start();
            }

            //
            //  Open Security event log.
            //
            EventLog secLog = new EventLog("Security");

            //
            //  Register event handler and enable raising events.
            //
            secLog.EntryWritten += new EntryWrittenEventHandler(OnEntryWritten);
            secLog.EnableRaisingEvents = true;
        }

        protected override void OnStop()
        {
            // 
            //  Exit the thread loop.
            //
            m_fLoop = false;
        }

        /// <summary>
        /// Loops until the service is stopped.
        /// </summary>
        private void ThreadLoop()
        {
            while (m_fLoop)
            {
                Thread.Sleep(1000);
            }
        }

        /// <summary>
        /// Handles the EntryWritten event of the Security EventLog. 
        /// </summary>
        /// <param name="source">The source of the event.</param>
        /// <param name="e">Contains the event data. </param>
        private void OnEntryWritten(object source, EntryWrittenEventArgs e)
        {
            XmlUpdateRequest xmlUpdateReq = new XmlUpdateRequest();

            Guid entryID = Guid.NewGuid();
            xmlUpdateReq.AddNode("Title", string.Format("Security Audit;{0};{1}", e.Entry.MachineName, entryID));
            xmlUpdateReq.AddNode("AuditStatus", (e.Entry.ReplacementStrings.Length > 7) ? e.Entry.ReplacementStrings[7] : string.Empty);
            xmlUpdateReq.AddNode("AuditSubStatus", (e.Entry.ReplacementStrings.Length > 9) ? e.Entry.ReplacementStrings[9] : string.Empty);
            xmlUpdateReq.AddNode("AuthenticationPackageName", (e.Entry.ReplacementStrings.Length > 12) ? e.Entry.ReplacementStrings[12] : string.Empty);
            xmlUpdateReq.AddNode("FailureReason", (e.Entry.ReplacementStrings.Length > 8) ? e.Entry.ReplacementStrings[8] : string.Empty);
            xmlUpdateReq.AddNode("LogonProcessName", (e.Entry.ReplacementStrings.Length > 11) ? e.Entry.ReplacementStrings[11] : string.Empty);
            xmlUpdateReq.AddNode("LogonType", (e.Entry.ReplacementStrings.Length > 10) ? e.Entry.ReplacementStrings[10] : string.Empty);
            xmlUpdateReq.AddNode("ProcessName", (e.Entry.ReplacementStrings.Length > 18) ? e.Entry.ReplacementStrings[18] : string.Empty);
            xmlUpdateReq.AddNode("SubjectDomainName", (e.Entry.ReplacementStrings.Length > 2) ? e.Entry.ReplacementStrings[2] : string.Empty);
            xmlUpdateReq.AddNode("SubjectLogonId", (e.Entry.ReplacementStrings.Length > 3) ? e.Entry.ReplacementStrings[3] : string.Empty);
            xmlUpdateReq.AddNode("SubjectUserName", (e.Entry.ReplacementStrings.Length > 1) ? e.Entry.ReplacementStrings[1] : string.Empty);
            xmlUpdateReq.AddNode("SubjectUserSid", (e.Entry.ReplacementStrings.Length > 0) ? e.Entry.ReplacementStrings[0] : string.Empty);
            xmlUpdateReq.AddNode("TargetDomainName", (e.Entry.ReplacementStrings.Length > 6) ? e.Entry.ReplacementStrings[6] : string.Empty);
            xmlUpdateReq.AddNode("TargetUserName", (e.Entry.ReplacementStrings.Length > 5) ? e.Entry.ReplacementStrings[5] : string.Empty);
            xmlUpdateReq.AddNode("TargetUserSid", (e.Entry.ReplacementStrings.Length > 4) ? e.Entry.ReplacementStrings[4] : string.Empty);
            xmlUpdateReq.AddNode("WorkstationName", (e.Entry.ReplacementStrings.Length > 13) ? e.Entry.ReplacementStrings[13] : string.Empty);
            xmlUpdateReq.AddNode("IpAddress", (e.Entry.ReplacementStrings.Length > 19) ? e.Entry.ReplacementStrings[19] : string.Empty);
            xmlUpdateReq.AddNode("IpPort", (e.Entry.ReplacementStrings.Length > 20) ? e.Entry.ReplacementStrings[20] : string.Empty);
            xmlUpdateReq.AddNode("MachineName", e.Entry.MachineName);
            xmlUpdateReq.AddNode("EventSource", e.Entry.Source);
            xmlUpdateReq.AddNode("EventCategory", e.Entry.Category);
            xmlUpdateReq.AddNode("EntryType", Convert.ToString(e.Entry.EntryType));
            xmlUpdateReq.AddNode("EventID", Convert.ToString(e.Entry.InstanceId));
            xmlUpdateReq.AddNode("TimeGenerated", e.Entry.TimeGenerated.ToString());
            xmlUpdateReq.AddNode("RawData", e.Entry.Message);

            SubmitUpdateRequest(xmlUpdateReq);
        }

        /// <summary>
        /// Sends an UpdateRequest XML document to the SPPdateModule via Named Pipes.
        /// </summary>
        /// <param name="xmlDoc">The UpdateRequest XML document.</param>
        private void SubmitUpdateRequest(XmlUpdateRequest xmlUpdateReq)
        {
            try
            {
                using (NamedPipeClientStream pipeClient = new NamedPipeClientStream(".", "SecurityEventReceiverPipe", PipeDirection.Out))
                {
                    //
                    // Connect to the pipe or wait for up to 30 seconds until the pipe is available.
                    //
                    pipeClient.Connect(30000);

                    //
                    //  Send the XML in a single base64 encoded line.
                    //
                    byte[] bytes = new ASCIIEncoding().GetBytes(xmlUpdateReq.OuterXmlBase64Encoded);
                    pipeClient.Write(bytes, 0, bytes.Length);
                }
            }
            catch (System.TimeoutException)
            {
                EventLog.WriteEntry(this.ServiceName, "Unable to connect to SecurityEventReceiverPipe. Make sure the SecurityEventReceiver service is running.", EventLogEntryType.Error, 1);
            }
            catch (Exception ee)
            {
                EventLog.WriteEntry(this.ServiceName, ee.Message, EventLogEntryType.Error, 2);
            }
        }
    }
}
