﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Microsoft.SharePoint.StsAdmin;
using Microsoft.SharePoint.Administration;
using System.Collections.Specialized;

namespace SecurityEventManagement
{
    public class SecurityEventServiceAdministration : ISPStsadmCommand
    {
        public string GetHelpMessage(string command)
        {
            System.Text.StringBuilder sbOut = new StringBuilder();

            sbOut.Append("\r\nInstalls/provisions or deprovisions/uninstalls Security Audit services.");
            sbOut.Append("\r\n\r\nParameters:\r\n");
            sbOut.Append("\r\n  -install     Installs the Security Audit services in the local farm.");
            sbOut.Append("\r\n  -provision   Creates service instances on the local SharePoint server.");
            sbOut.Append("\r\n  -deprovision Removes the service instances from the local server.");
            sbOut.Append("\r\n  -uninstall   Removes all service instances and services from the local farm.");
            sbOut.Append("\r\n\r\nExamples:\r\n");
            sbOut.Append("\r\n  stsadm -o securityevents -install -provision");
            sbOut.Append("\r\n  stsadm -o securityevents -uninstall");
            return sbOut.ToString();
        }

        public int Run(string command, StringDictionary keyValues, out string output)
        {
            output = string.Empty;

            try
            {
                if (keyValues.ContainsKey("install"))
                {
                    output += Environment.NewLine
                        + InstallSecurityEventHandler()
                        + Environment.NewLine
                        + InstallSecurityEventReceiver();
                }

                if (keyValues.ContainsKey("provision"))
                {
                    output += Environment.NewLine
                        + ProvisionSecurityEventHandler()
                        + Environment.NewLine
                        + ProvisionSecurityEventReceiver();
                }

                if (keyValues.ContainsKey("deprovision"))
                {
                    output += Environment.NewLine
                        + DeProvisionSecurityEventHandler()
                        + Environment.NewLine
                        + DeProvisionSecurityEventReceiver();
                }

                if (keyValues.ContainsKey("uninstall"))
                {
                    output += Environment.NewLine
                        + UnInstallSecurityEventHandler()
                        + Environment.NewLine
                        + UnInstallSecurityEventReceiver();
                }

                return 1;
            }
            catch(Exception ee)
            {
                output = ee.Message;
                return 0;
            }
        }
        private string InstallSecurityEventHandler()
        {
            //
            //  Let's see if the farm already has a SecurityEventHandlerService.
            //
            SecurityEventHandlerService EventHandlerService =
                SPFarm.Local.Services.GetValue<SecurityEventHandlerService>(
                SecurityEventHandlerService.ntServiceName);
            if (EventHandlerService == null)
            {
                //
                //  No, so create the SecurityEventHandlerService.
                //
                EventHandlerService = new SecurityEventHandlerService(SPFarm.Local);
                EventHandlerService.Update();
                return "EventHandlerService successfully created.";
            }
            else
            {
                return "EventHandlerService already exists.";
            }
        }
        private string InstallSecurityEventReceiver()
        {
            //
            //  Let's see if the farm already has a SecurityEventReceiverService.
            //
            SecurityEventReceiverService EventReceiverService =
                SPFarm.Local.Services.GetValue<SecurityEventReceiverService>(
                SecurityEventReceiverService.ntServiceName);
            if (EventReceiverService == null)
            {
                //
                //  Create the SecurityEventReceiverService.
                //
                EventReceiverService = new SecurityEventReceiverService(SPFarm.Local);
                EventReceiverService.Update();
                return "SecurityEventReceiverService successfully created.";
            }
            else
            {
                return "SecurityEventReceiverService already exists.";
            }
        }

        private string ProvisionSecurityEventHandler()
        {
            //
            //  Check if the local server already has a SecurityEventHandlerServiceInstance.
            //
            SecurityEventHandlerServiceInstance EventHandlerInstance =
                SPServer.Local.ServiceInstances.GetValue<SecurityEventHandlerServiceInstance>(
                SecurityEventHandlerServiceInstance.serviceInstanceName);
            if (EventHandlerInstance != null)
                return "SecurityEventHandlerServiceInstance already exists.";

            //
            //  Next, check to see if the farm knows about the SecurityEventHandlerService.
            //
            SecurityEventHandlerService EventHandlerService =
                SPFarm.Local.Services.GetValue<SecurityEventHandlerService>(
                SecurityEventHandlerService.ntServiceName);
            if (EventHandlerService == null)
            {
                //
                //  Need to create the SecurityEventHandlerService first.
                //
                throw new Exception("You must use the -install option to create the SecurityEventHandlerService before you can provision a SecurityEventHandlerServiceInstance.");
            }

            //
            //  Now create the SecurityEventHandlerServiceInstance on the local server.
            //
            EventHandlerInstance = new SecurityEventHandlerServiceInstance(EventHandlerService);
            EventHandlerInstance.Update();
            return "SecurityEventHandlerServiceInstance successfully created.";
        }
        private string ProvisionSecurityEventReceiver()
        {
            //
            //  Check if the local server already has a SecurityEventReceiverServiceInstance.
            //
            SecurityEventReceiverServiceInstance EventReceiverInstance =
                SPServer.Local.ServiceInstances.GetValue<SecurityEventReceiverServiceInstance>(
                SecurityEventReceiverServiceInstance.serviceInstanceName);
            if (EventReceiverInstance != null)
                return "SecurityEventReceiverServiceInstance already exists.";

            //
            //  Next, check to see if the farm knows about the SecurityEventReceiverService.
            //
            SecurityEventReceiverService EventReceiverService =
                SPFarm.Local.Services.GetValue<SecurityEventReceiverService>(
                SecurityEventReceiverService.ntServiceName);
            if (EventReceiverService == null)
            {
                //
                //  Need to create the SecurityEventReceiverServiceInstance first.
                //
                throw new Exception("You must use the -install option to create the SecurityEventHandlerService before you can provision a SecurityEventReceiverServiceInstance.");
            }
            
            //
            //  Now create the SecurityEventReceiverServiceInstance on the local server.
            //
            EventReceiverInstance = new SecurityEventReceiverServiceInstance(EventReceiverService);
            EventReceiverInstance.Update();
            return "SecurityEventReceiverServiceInstance successfully created.";
        }

        private string DeProvisionSecurityEventHandler()
        {
            //
            //  Get the local SecurityEventHandlerServiceInstance.
            //
            SecurityEventHandlerServiceInstance EventHandlerInstance =
                SPServer.Local.ServiceInstances.GetValue<SecurityEventHandlerServiceInstance>(
                SecurityEventHandlerServiceInstance.serviceInstanceName);
            if (EventHandlerInstance != null)
            {
                EventHandlerInstance.Unprovision();
                EventHandlerInstance.Delete();
                return "SecurityEventHandlerServiceInstance successfully deprovisioned.";
            }
            else
            {
                return "Unable to locate local SecurityEventHandlerServiceInstance.";
            }
        }
        private string DeProvisionSecurityEventReceiver()
        {
            //
            //  Check if the local server already has a SecurityEventReceiverServiceInstance.
            //
            SecurityEventReceiverServiceInstance EventReceiverInstance =
                SPServer.Local.ServiceInstances.GetValue<SecurityEventReceiverServiceInstance>(
                SecurityEventReceiverServiceInstance.serviceInstanceName);
            if (EventReceiverInstance != null)
            {
                EventReceiverInstance.Unprovision();
                EventReceiverInstance.Delete();
                return "SecurityEventReceiverServiceInstance successfully deprovisioned.";
            }
            else
            {
                return "Unable to locate local SecurityEventReceiverServiceInstance.";
            }
        }

        private string UnInstallSecurityEventHandler()
        {
            //
            //  Get the farm's SecurityEventHandlerService.
            //
            SecurityEventHandlerService EventHandlerService =
                SPFarm.Local.Services.GetValue<SecurityEventHandlerService>(
                SecurityEventHandlerService.ntServiceName);
            if (EventHandlerService == null)
            {
                return "Unable to locate SecurityEventHandlerService.";
            }

            //
            //  Remove all existing service instances.
            //
            foreach (SecurityEventHandlerServiceInstance ServiceInstance in EventHandlerService.Instances)
            {
                ServiceInstance.Unprovision();
                ServiceInstance.Delete();
            }

            EventHandlerService.Unprovision();
            EventHandlerService.Delete();

            return "All SecurityEventHandlerServiceInstance and SecurityEventHandlerService objects successfully removed.";
        }
        private string UnInstallSecurityEventReceiver()
        {
            //
            //  Let's see if the farm already has a SecurityEventReceiverService.
            //
            SecurityEventReceiverService EventReceiverService =
                SPFarm.Local.Services.GetValue<SecurityEventReceiverService>(
                SecurityEventReceiverService.ntServiceName);
            if (EventReceiverService == null)
            {
                throw new Exception("Unable to locate SecurityEventReceiverService.");
            }

            //
            //  Remove all existing service instances.
            //
            foreach (SecurityEventReceiverServiceInstance ServiceInstance 
                in EventReceiverService.Instances)
            {
                ServiceInstance.Unprovision();
                ServiceInstance.Delete();
            }

            EventReceiverService.Unprovision();
            EventReceiverService.Delete();

            return "All SecurityEventReceiverServiceInstance and SecurityEventReceiverService objects successfully removed.";
        }
    }
}
