﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Microsoft.SharePoint.Administration;
using System.Text.RegularExpressions;
using System.DirectoryServices;

namespace SecurityEventManagement
{
    public class SecurityEventReceiverService : SPWindowsService
    {
        public const string ntServiceName = "SecurityEventReceiver";

        [Persisted]
        string m_SiteURL;
        [Persisted]
        string m_ListName;
        [Persisted]
        string m_EventFilter;

        public SecurityEventReceiverService()
        {
        }

        public SecurityEventReceiverService(SPFarm spFarm)
            : this(spFarm, "NT AUTHORITY\\NETWORK SERVICE", string.Empty)
        {
        }

        public SecurityEventReceiverService(SPFarm spFarm, 
            string Username)
            : this(spFarm, Username, string.Empty)
        {
        }

        public SecurityEventReceiverService(
            SPFarm spFarm, string Username, string Password)
            : base(ntServiceName, spFarm)
        {
            if (string.IsNullOrEmpty(Username))
                throw new ArgumentException(
                    "Unable to create SecurityEventReceiverService " +
                    "due to missing security account name.");

            switch (Username.ToUpper())
            {
                case "NT AUTHORITY\\SYSTEM":
                    base.ProcessIdentity.CurrentIdentityType = 
                        IdentityType.LocalSystem;
                    break;
                case "NT AUTHORITY\\LOCAL SERVICE":
                    base.ProcessIdentity.CurrentIdentityType = 
                        IdentityType.LocalService;
                    break;
                case "NT AUTHORITY\\NETWORK SERVICE":
                    base.ProcessIdentity.CurrentIdentityType = 
                        IdentityType.NetworkService;
                    break;
                default:
                    //
                    // User name must be a NetBIOS login name.
                    //
                    Regex oRex = new Regex(@"^([a-z][a-z0-9.-]{1,14})\\" +
                        @"(?!\s.+$)([^][\\/"":|<>+=;,?*@]{1,15})$",
                        RegexOptions.IgnoreCase);
                    if(!oRex.IsMatch(Username))
                        throw new ArgumentException(
                            "Unable to create SecurityEventReceiverService " +
                            "due to invalid NetBIOS account name.");

                    //
                    //  Verify credentials by logging on to Active Directory.
                    //
                    try
                    {
                        string[] login = Username.Split(new char[] { '\\' }, 2);
                        string domainName = login[0];
                        string userName = login[1];

                        if(new DirectoryEntry("LDAP://" + domainName,
                                userName, Password).NativeObject != null)
                        {
                            //
                            //  Credentials OK, set the user account information.
                            //
                            base.ProcessIdentity.CurrentIdentityType =
                                IdentityType.SpecificUser;
                            base.ProcessIdentity.Username = Username;
                            base.ProcessIdentity.Password = Password;
                        }

                    }
                    catch
                    {
                        throw new ArgumentException(
                            "Unable to create SecurityEventReceiverService " +
                            "due to invalid security account credentials.");
                    }
                    break;
            }

            base.ProcessIdentity.IsCredentialDeploymentEnabled = true;
            base.ProcessIdentity.IsCredentialUpdateEnabled = true;

        }

        public string SiteURL
        {
            get
            {
                return (string.IsNullOrEmpty(m_SiteURL)) ? "http://wss" : m_SiteURL;
            }
            set
            {
                m_SiteURL = value;
            }
        }

        public string ListName
        {
            get
            {
                return (string.IsNullOrEmpty(m_ListName)) ? "Security Audit" : m_ListName;
            }
            set
            {
                m_ListName = value;
            }
        }

        public string EventFilter
        {
            get
            {
                return (string.IsNullOrEmpty(m_EventFilter)) ? "FailureAudit" : m_EventFilter;
            }
            set
            {
                m_EventFilter = value;
            }
        }
    }
}
