﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.ServiceProcess;
using System.Text;

using System.Xml;
using Microsoft.SharePoint;
using System.Threading;

using System.IO;
using System.IO.Pipes;
using System.Security.AccessControl;
using Microsoft.Win32;

using Microsoft.SharePoint.Administration;
using SecurityEventManagement;

namespace SecurityEventReceiver
{
    public partial class SecurityEventReceiver : ServiceBase
    {
        bool m_fLoop = true;
        Thread m_SvcThread = null;
        NamedPipeServerStream m_PipeServer = null;

        public SecurityEventReceiver()
        {
            InitializeComponent();
        }

        protected override void OnStart(string[] args)
        {
            //
            //  Create an access rule for the system account.
            //
            PipeSecurity pipeSecurity = new PipeSecurity();
            PipeAccessRule accessRule = new PipeAccessRule(@"NT AUTHORITY\SYSTEM",
                PipeAccessRights.FullControl,
                AccessControlType.Allow);
            pipeSecurity.AddAccessRule(accessRule);

            //
            //  Create an inbound named pipe with custom pipe security settings
            //  that only grant the System account access permissions.
            //
            if (m_PipeServer == null)
            {
                m_PipeServer = new NamedPipeServerStream("SecurityEventReceiverPipe", PipeDirection.In,
                1, PipeTransmissionMode.Message, PipeOptions.Asynchronous, 4096, 4096, pipeSecurity);
                m_PipeServer.BeginWaitForConnection(new AsyncCallback(WaitForClientConnection), null);
            }

            //
            //  Start a worker thread.
            //
            if (m_SvcThread == null)
            {
                m_SvcThread = new Thread(ThreadLoop);
                m_SvcThread.Start();
            }
        }

        protected override void OnStop()
        {
            try
            {
                //
                //  Exit the thread loop.
                //
                m_fLoop = false;

                //
                //  Close the pipe.
                //
                if (m_PipeServer != null)
                {
                    m_PipeServer.Close();
                    m_PipeServer = null;
                }
            }
            catch (Exception ee)
            {
                EventLog.WriteEntry(this.ServiceName, ee.Message, EventLogEntryType.Error, 1);
            }
        }

        /// <summary>
        /// Waits for SecurityEventHandler to connect
        /// and processes submitted requests.
        /// </summary>
        /// <param name="ar"></param>
        private void WaitForClientConnection(IAsyncResult ar)
        {
            Decoder decoder = Encoding.ASCII.GetDecoder();
            Byte[] bytes = new Byte[4096];
            Char[] chars = new Char[4096];

            try
            {
                m_PipeServer.EndWaitForConnection(ar);
                if (m_PipeServer.IsConnected)
                {
                    //
                    //  Get a command from the SecurityEventHandler process.
                    //
                    int numBytes;
                    string cmdLine = string.Empty;
                    do
                    {
                        do
                        {
                            numBytes = m_PipeServer.Read(bytes, 0, bytes.Length);
                            int numChars = decoder.GetChars(bytes, 0, numBytes, chars, 0);
                            cmdLine += new String(chars, 0, numChars);

                        } while (!m_PipeServer.IsMessageComplete);
                        decoder.Reset();
                    } while (numBytes != 0);

                    SPUpdateRequest spUpdateRequest = new SPUpdateRequest(cmdLine);
                    spUpdateRequest.AddEntry();

                    m_PipeServer.Disconnect();
                }
                m_PipeServer.BeginWaitForConnection(new AsyncCallback(WaitForClientConnection), null);
            }
            catch (ObjectDisposedException) { }
            catch (Exception ee)
            {
                EventLog.WriteEntry(this.ServiceName, ee.Message, EventLogEntryType.Error, 2);
            }
        }

        private void ThreadLoop()
        {
            while (m_fLoop)
            {
                Thread.Sleep(1000);
            }
        }
    }

    class SPUpdateRequest
    {
        XmlDocument m_XmlRequest = null;

        /// <summary>
        /// Load an XmlDocument with the Update request from a base64 encoded command line.
        /// </summary>
        /// <param name="pipeCmdLine">The base64 encoded command line.</param>
        public SPUpdateRequest(string pipeCmdLine)
        {
            try
            {
                m_XmlRequest = new XmlDocument();
                m_XmlRequest.LoadXml(base64_unicode_decode(pipeCmdLine));
            }
            catch { m_XmlRequest = null; }
        }

        /// <summary>
        /// The actual function to do the work.
        /// </summary>
        public void AddEntry()
        {
            try
            {
                if (m_XmlRequest == null ||
                    m_XmlRequest.DocumentElement.Name != "UpdateRequest") return;

                AddEntryToSPList();
            }
            catch (Exception ee)
            {
                EventLog.WriteEntry("SecurityEventReceiver", ee.Message, EventLogEntryType.Error, 3);
            }
        }

        /// <summary>
        /// Helper function to decode a base64-encoded Unicode string.
        /// </summary>
        /// <param name="base64String">The base64-encoded Unicode string.</param>
        /// <returns>The decoded string in clear text.</returns>
        private string base64_unicode_decode(string base64String)
        {
            if (string.IsNullOrEmpty(base64String))
                return string.Empty;

            UnicodeEncoding ue = new UnicodeEncoding();
            return ue.GetString(Convert.FromBase64String(base64String));
        }

        /// <summary>
        /// Helper function to return the inner text of an XML child node.
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        private string NodeValue(string name)
        {
            XmlNode node = m_XmlRequest.SelectSingleNode("UpdateRequest/" + name);
            return (node != null) ? node.InnerText : string.Empty;
        }

        /// <summary>
        /// Parses the XmlUpdateRequest document of the current class instance 
        /// and creates a corresponding item in a SharePoint list.
        /// </summary>
        private void AddEntryToSPList()
        {
            /*******************************************************************************************/
            // Read service parameters from the configuration database instead of the local registry.
            /*******************************************************************************************/
            string siteURL = "http://wss";
            string listName = "Security Audit";
            string eventFilter = "FailureAudit"; // "FailureAudit|SuccessAudit";

            try
            {
                //string regKey = @"SYSTEM\CurrentControlSet\Services\SecurityEventReceiver\Parameters";
                //using (RegistryKey svcParams = Registry.LocalMachine.OpenSubKey(regKey))
                //{
                //    siteURL = Convert.ToString(svcParams.GetValue("SiteURL"));
                //    listName = Convert.ToString(svcParams.GetValue("ListName"));
                //    eventFilter = Convert.ToString(svcParams.GetValue("EventFilter"));
                //}
                SecurityEventReceiverService EventReceiverService =
                    SPFarm.Local.Services.GetValue<SecurityEventReceiverService>(
                    SecurityEventReceiverService.ntServiceName);

                if (EventReceiverService == null)
                    throw new Exception("Unable to locate SecurityEventReceiverService "
                                        + "in the local SharePoint farm.");

                siteURL = EventReceiverService.SiteURL;
                listName = EventReceiverService.ListName;
                eventFilter = EventReceiverService.EventFilter;

            }
            catch (Exception ee)
            {
                EventLog.WriteEntry("SecurityEventReceiver", ee.Message, EventLogEntryType.Error, 3);
                return;
            }
            /*******************************************************************/

            if (!eventFilter.Contains(NodeValue("EntryType")))
            {
                EventLog.WriteEntry("SecurityEventReceiver", "Item skipped due to event filter settings.");
                return;
            }

            try
            {
                //
                //  Access the site and create the item in the specified list.
                //
                using (SPSite site = new SPSite(siteURL))
                using (SPWeb web = site.OpenWeb())
                {
                    SPList list = web.Lists[listName];
                    SPListItem item = list.Items.Add(list.RootFolder.ServerRelativeUrl, SPFileSystemObjectType.Folder);

                    item["Content Type"] = "SecurityAudit";
                    item["Title"] = NodeValue("Title");
                    item["AuditStatus"] = NodeValue("AuditStatus");
                    item["AuditSubStatus"] = NodeValue("AuditSubStatus");
                    item["AuthenticationPackageName"] = NodeValue("AuthenticationPackageName");
                    item["FailureReason"] = NodeValue("FailureReason");
                    item["LogonProcessName"] = NodeValue("LogonProcessName");
                    item["LogonType"] = NodeValue("LogonType");
                    item["ProcessName"] = NodeValue("ProcessName");
                    item["SubjectDomainName"] = NodeValue("SubjectDomainName");
                    item["SubjectLogonId"] = NodeValue("SubjectLogonId");
                    item["SubjectUserName"] = NodeValue("SubjectUserName");
                    item["SubjectUserSid"] = NodeValue("SubjectUserSid");
                    item["TargetDomainName"] = NodeValue("TargetDomainName");
                    item["TargetUserName"] = NodeValue("TargetUserName");
                    item["TargetUserSid"] = NodeValue("TargetUserSid");
                    item["WorkstationName"] = NodeValue("WorkstationName");
                    item["IpAddress"] = NodeValue("IpAddress");
                    item["IpPort"] = NodeValue("IpPort");
                    item["MachineName"] = NodeValue("MachineName");
                    item["EventSource"] = NodeValue("EventSource");
                    item["EventCategory"] = NodeValue("EventCategory");
                    item["EntryType"] = NodeValue("EntryType");
                    item["EventID"] = NodeValue("EventID");
                    item["TimeGenerated"] = NodeValue("TimeGenerated");
                    item["RawData"] = NodeValue("RawData");
                    item.Update();
                }
                EventLog.WriteEntry("SecurityEventReceiver", string.Format("Entry '{0}' added successully to SharePoint list '{1}'.", NodeValue("Title"), listName));
            }
            catch (Exception ee)
            {
                EventLog.WriteEntry("SecurityEventReceiver", ee.Message, EventLogEntryType.Error, 5);
            }
        }
    }
}
